#!/bin/bash
# (c) Copyright 2007-2014 Hewlett Packard Development Company, L.P.
# Version A.12.00.00

############################################################################
# HA MySQL Toolkit Main Script
############################################################################
#
# This script  contains  MySQL's  defined  functions that are used in the
# package toolkit control script which is used by Package Control Script.
#
# To be a part of a package,  this script has to be placed in the package
# directory.  The toolkit  control  script will check for this  script in
# the package  directory.  If it exists, the toolkit  control script will
# call it.
#
# Exit value  definition:  1 - a failure  occurs in one of the  function.
# Return control to the toolkit control script.
#
# REVISION HISTORY:
#   Version      mm/dd/yyyy   Changes
#   A.03.00      07/12/2006   1) Removed code related to SELinux. Refer README
#                                for more details.
#                             2) Fixed the bug in stop_mysql_server that was
#                                exiting before removing MySQL processes completely
#                             3) Added code related to maintenance mode
#                07/26/2006   4) Added code in stop_mysql_server to read pid from
#                                temporary file if pid file is not present.
#   A.03.01      07/25/2007   1) Issue related to the stale pid file resolved.
#
#   A.04.02.01   04/30/2012   1) Removed check for the attribute DATA_DIRECTORY. 
#                             2) Added check for MONITOR_INTERVAL.
#
#   A.04.02.02   10/19/2012   1) Added mysqladmin command to stop the MySQL server
#
#   A.12.00.00   09/15/2013   1) Removed toolkit maintenance mode support
#                             2) Removed Legacy style packaging support
#                             3) Added idempotent operation support
##############################################################################



######################### VARIABLES & PARAMETERS ########################
#
# Variables      : Initialize the variables with suitable values. 
#                  
# PKG_DIR= <Path of Package & toolkit configuration & control files>
#
#
# STARTUP_RETRIES = < Number > :
#
#                 This  variable  specifies  number  of  retries  in case
#                 pid-file.  is not found while  monitoring  or  startup.
#                 Default is set to "10" retries.
#
# exit_code = < Initial Non-Error State >
#
#                 This  variable  contain the exit status of this script.
#                 It  is  initially  assigned  "0",  indication  no-error
#                 condition.
#
# Arguments       :
#
#
# $1 = Action     :
#                 First parameter,  specified the operation to be carried
#                 out viz.  "start",  "stop"  or  "monitor".  Appropriate
#                 function is called for each operation.
#
#
#  Usage :  ./hamysql.sh start | stop | monitor. 
#
#########################################################################

. /etc/cmcluster.conf
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin:$SGSBIN:

PKG_DIR=${0%/*}

STARTUP_RETRIES=10 

exit_code=0


# Assigning default value 30 to MONITOR_INTERVAL.
if [ -z $MONITOR_INTERVAL ]; then
    MONITOR_INTERVAL=30
fi



# Temporory file for storing pid of mysql process when mysql pid file is removed
TEMP_PID_FILE=""$SGCONF/$SG_PACKAGE"_mysqltmp.pid"

# Confirm that PID_FILE variable contains non-null string. 
if [ -z $PID_FILE ]; then
    PID_FILE=""$SGCONF/$SG_PACKAGE"_mysqltmp.pid"
fi

# Setting TIME_OUT default value if not defined
if [ -z "$TIME_OUT" ] ; then
    TIME_OUT=30
fi


 function find_proc
{
   #Check the /proc FS for the status of the Mysql daemon
   grep $1 /proc/$2/stat >/dev/null 2>&1
   return $?
}



######################### FUNCTION HEADER ###############################
# Function Name  : start_mysql_server
# In Parameters  : <Nil>
#
# Out   paramater: <Nil>
#                  
#
# Description    : Starts the MySQL server with suitable parameter.
#
#########################################################################

function start_mysql_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting MySQL daemons."

    local options=" --pid-file=$PID_FILE "

    if [ ! -z $CONFIGURATION_FILE_PATH ] && [ -f $CONFIGURATION_FILE_PATH ]; then
        options="--defaults-file=$CONFIGURATION_FILE_PATH  $options"
    elif [ ! -z $DATA_DIRECTORY ] && [ -d $DATA_DIRECTORY ]; then  
        options="--defaults-file=$DATA_DIRECTORY/my.cnf $options"  
    else 
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting ERROR: MySQL configuration file not specified."
        check_return 1 6
    fi

    #Checking for selinux enabled or disabled 
    selinuxenabled >/dev/null 2>&1
    IS_SE_ENABLED=`echo $?`

    # If SELinux is enabled then warn the user
    if [ $IS_SE_ENABLED -eq 0 ]
    then
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": Warning: SELinux is enabled"
    fi

    #If pid file is already present with a stale entry then its removed and if it has a valid entry then exit returning 1
    if [ -f $PID_FILE ]; then
      val=`cat $PID_FILE`
      find_proc 'mysqld' $val
      ret=$?
      if (test $ret = 0) then
        echo  "$(date '+%b %e %T') - Node \"$(hostname)\": MySQL deamons are already running."
        return 0
      else
        echo "$(date '+%b %e %T')- Node \"$(hostname)\": Warning: Stale pid file $PID_FILE found. Removing $PID_FILE "
        rm $PID_FILE
      fi
    fi


    #Start the MySQL DB Server
    mysqld_safe $options &
    ret_value=$?

    # Check the MySQL Startup Error Codes (Exit if Failed)
    check_return $ret_value 1

    #Check if the pid file is ready
    counter=0    
    while :; do
        if [ -f $PID_FILE ]; then
            break
        else
            if [ $counter -ge $STARTUP_RETRIES ]; then
                echo "$(date '+%b %e %T') - ERROR: Function start_myssql_server: mysql failed to create PID file $PID_FILE"
                check_return 1 1
            else
                sleep 1
                let counter="$counter+1"
            fi
        fi
    done
}

######################### FUNCTION HEADER ###############################
# Function Name  : stop_mysql_server
# In Parameters  : <Nil>
#
# Out   paramater: <Nil>
#                  
#
# Description    : Stops the MySQL server.
#
#########################################################################

function stop_mysql_server
{
   echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stopping MySQL daemons"
   mysqladmin --defaults-file=$CONFIGURATION_FILE_PATH shutdown
   if [ $? -eq 0 ]; then
     echo "$(date '+%b %e %T') - Successfully stopped the MySQL Server."
     if [ -f $TEMP_PID_FILE ]; then
        rm -f $TEMP_PID_FILE
     fi
   else 
   # If above command fails, check for any stale MySQL daemons and kill them.
    if [ -f $PID_FILE ]; then
        read pid < $PID_FILE
    elif [ -f $TEMP_PID_FILE ]; then
        read pid < $TEMP_PID_FILE
        rm -f $TEMP_PID_FILE
    else
        echo "$(date '+%b %e %T') - ERROR: Function stop_mysql_server: The $PID_FILE does not exist."
        check_return 1 4
        exit $exit_code
    fi

    if [ -n $pid ]; then
        #kill -s SIGTERM $pid #Does not seem to be working on RH4.
        kill -15 $pid
        return_value=$?
        if [ $return_value -eq 0 ]; then
           i=0
           # Wait until Mysql process is removed completely
           while [ $i -lt $TIME_OUT ]
           do
              if [ -d /proc/$pid ];then
                 sleep 1
              else
                 break
              fi
              let i="$i+1"
           done
        fi
        if [ -d /proc/$pid ];then
           kill -9 $pid
           return_value=$?
        fi
           check_return $return_value 4
    else
        echo "$(date '+%b %e %T') - ERROR: Function stop_mysql_server: The $PID_FILE was corrupted."
        check_return 1 4
    fi
  fi
}


######################### FUNCTION HEADER ###############################
# Function Name  : check_return
# In Parameters  : 
#
#             $1 : Exit Status of previously run command. 
#
#             $2 : Message Number to be printed.
#
# Out   paramater: <Nil>
#                  
# Description    : Check return value of functions and log appropriate 
#                  message before exiting.
#
#########################################################################

function check_return
{
    if (( $1 != 0 ))
    then
    case $2 in
        1)
	     echo "$(date '+%b %e %T') - ERROR: Function start_mysql_server: Failed to start HA MySQL daemons."
	     echo "$(date '+%b %e %T') - Please check Mysql log files for more details."
             stop_mysql_server
             exit 1
        ;;
        2)
             echo "$(date '+%b %e %T') - ERROR: Function start_mysql_services: Failed to start HA MySQL services $I"
             halt_mysql_services
             stop_mysql_server
             exit 1
        ;;
        3)
             echo "$(date '+%b %e %T') - ERROR: Function halt_mysql_services: Failed to halt HA MySQL services $I"
             exit_code=0
        ;;
        4)
             echo "$(date '+%b %e %T') - ERROR: Function stop_mysql_server: Failed to stop MySQL daemons"
             echo "$(date '+%b %e %T') - Make sure to kill all lingering processes of this MySQL package"
             exit_code=0
        ;;
        5)   echo "$(date '+%b %e %T') - ERROR: Could not Start the Monitor Function"
             exit 1
        ;;
        6)   echo "$(date '+%b %e %T') - ERROR: Improper parameters in Toolkit Configuration file"
             exit 1
        ;;
        *)
             echo "$(date '+%b %e %T') - ERROR: Failed, unknown error."
             exit 1
             ;;
    esac
    fi
}


######################### STARTING POINT ###############################
#
# Description    : Invoke the Toolkit by calling appropriate function 
#                  depending on the input parameter.
#
# ----------------------------------------------------------------------
#  PARAMETER     |    Function Name 
# ----------------------------------------------------------------------
#   start        | start_mysql_server
#                |
#   stop         | stop_mysql_server
#                | 
#   monitor      | Call hamysql.mon Script 
# ----------------------------------------------------------------------
#
#########################################################################


 case $1 in
 start)
    start_mysql_server
 ;;

 stop)
    stop_mysql_server
 ;;

 monitor)
    . $TKIT_SCRIPT_DIR/hamysql.mon $PID_FILE $MONITOR_INTERVAL $RETRY_TIMES
    ret_value=$?
    check_return ret_value 5
 ;;
 
 *)
 echo -e "$(date '+%b %e %T') - Invalid Paramter specified\n"
 echo -e "\t\t- Usage:  ${0} [ start | stop | monitor ] "
 ;;
 esac

exit $exit_code
